#if !defined(RT_MESH_H)
#define RT_MESH_H
//----------------------------------------------------------
// mesh.h
//
// ID3DXMesh locking helper classes like <rt/vertexbuf.h>.
//

namespace rt
{

//----------------------------------------------------------
// mesh_vertex_lock<>
//
// Lock the vertex buffer of a mesh
//
template <typename Vertex>
class mesh_vertex_lock
{
public:
    mesh_vertex_lock(ID3DXMesh *mesh, DWORD flags = 0)
        : m_mesh(mesh),
        m_data(0)
    {
        BYTE *data = 0;
        THR(m_mesh->LockVertexBuffer(flags, &data));
        m_data = reinterpret_cast<Vertex *>(data);
    }
    ~mesh_vertex_lock()
    {
        const HRESULT hr = m_mesh->UnlockVertexBuffer();
        ATLASSERT(SUCCEEDED(hr));
    }

    const Vertex *data() const  { return m_data; }
    Vertex *data()              { return m_data; }

private:
    CComPtr<ID3DXMesh> m_mesh;
    Vertex *m_data;
};

//----------------------------------------------------------
// mesh_index_lock<>
//
// Lock the index buffer of a mesh.
//
template <typename Index>
class mesh_index_lock
{
    mesh_index_lock(ID3DXMesh *mesh, DWORD flags = 0)
        : m_mesh(mesh),
        m_data(0),
    {
        BYTE *data = 0;
        THR(m_mesh->LockIndexBuffer(flags, &data));
        m_data = reinterpret_cast<Index *>(data);
    }
    ~mesh_index_lock()
    {
        const HRESULT hr = m_mesh->UnlockIndexBuffer();
        ATLASSERT(SUCCEEDED(hr));
    }

    const Index *data() const   { return m_data; }
    Index *data()               { return m_data; }

private:
    CComPtr<ID3DXMesh> m_mesh;
    Index *m_data;
};

//-----------------------------------------------------------------------------
// dx_buffer<>
//
// This template class makes an ID3DXBuffer's data pointer look like a T *,
// not a BYTE *.
//
// 'p' is the interface pointer public member in CComPtr<> (see <atlbase.h>)
//
template <typename T>
class dx_buffer : public CComPtr<ID3DXBuffer>
{
public:
    dx_buffer() : CComPtr<ID3DXBuffer>() {}
    dx_buffer(UINT count) : CComPtr<ID3DXBuffer>()
    {
        THR(::D3DXCreateBuffer(count*sizeof(T),
            CComPtr<ID3DXBuffer>::operator&()));
    }
    operator const T *() const
    {
        ATLASSERT(p);
        return reinterpret_cast<T *>(p->GetBufferPointer());
    }
    operator T *()
    {
        ATLASSERT(p);
        return reinterpret_cast<T *>(p->GetBufferPointer());
    }
};

};

#endif
