#if !defined(RT_VERTEXBUF_H)
#define RT_VERTEXBUF_H
//----------------------------------------------------------
// vertexbuf.h
//
// Helper classes for use with vertex buffers.
//
#include <atlbase.h>

namespace rt {

//----------------------------------------------------------
// vertex_lock
//
// Locks a vertex buffer in its constructor and unlocks it
// in its destructor, providing for exception safe access
// to vertex data.  Takes a template argument Vertex that
// is the type of the underlying vertices to be locked.
//
template <typename Vertex>
class vertex_lock
{
public:
    // lock in constructor, unlock in destructor
    vertex_lock(IDirect3DVertexBuffer8 *vb,
        DWORD flags = 0,
        UINT offset = 0,
        UINT size = 0)
        : m_vb(vb)
    {
        BYTE *data = 0;
        THR(m_vb->Lock(offset, size, &data, flags));
        m_data = reinterpret_cast<Vertex *>(data);
    }
    ~vertex_lock()
    {
        const HRESULT hr = m_vb->Unlock();
        ATLASSERT(SUCCEEDED(hr));
    }

    // type safe accessors to vertex data
    const Vertex *data() const { return m_data; }
    Vertex *data() { return m_data; }

private:
    CComPtr<IDirect3DVertexBuffer8> m_vb;
    Vertex *m_data;
};

}; // rt

#endif
