//-----------------------------------------------------------------------------
// File: rt_FrameBuffer.h
//
// Desc: Header file rt_FrameBuffer sample app
//-----------------------------------------------------------------------------
#ifndef AFX_RT_FRAMEBUFFER_H__9028BB79_8A32_4ED4_9BE6_FF4CFAA40EFE__INCLUDED_
#define AFX_RT_FRAMEBUFFER_H__9028BB79_8A32_4ED4_9BE6_FF4CFAA40EFE__INCLUDED_

#include <vector>

#include <atlbase.h>

#include <d3d8.h>

#include "D3DApp.h"
#include "D3DFont.h"

#include "rt/tstring.h"

//-----------------------------------------------------------------------------
// Defines, and constants
//-----------------------------------------------------------------------------
#define DXAPP_KEY        TEXT("Software\\Pahvant\\rt_FrameBuffer")

extern const UINT TILE_SIZE;
extern const UINT STIPPLE_SIZE;
extern const float *g_jitter[16];

// Struct to store the current input state
struct UserInput
{
    UserInput()
        : bRotateUp(FALSE),
        bRotateDown(FALSE),
        bRotateLeft(FALSE),
        bRotateRight(FALSE)
    {}
    BOOL bRotateUp;
    BOOL bRotateDown;
    BOOL bRotateLeft;
    BOOL bRotateRight;
};

struct s_blend_factor
{
    DWORD m_src;
    DWORD m_dest;
};

extern const s_blend_factor g_blend_factors[12];

// this enumeration must be in the same order as the menu IDs in resource.h
enum e_blending_operator
{
    E_BLENDING_CLEAR = 0,
    E_BLENDING_SRC,
    E_BLENDING_DEST,
    E_BLENDING_SRC_OVER_DEST,
    E_BLENDING_DEST_OVER_SRC,
    E_BLENDING_SRC_IN_DEST,
    E_BLENDING_DEST_IN_SRC,
    E_BLENDING_SRC_ATOP_DEST,
    E_BLENDING_DEST_ATOP_SRC,
    E_BLENDING_SRC_OUT_DEST,
    E_BLENDING_DEST_OUT_SRC,
    E_BLENDING_SRC_XOR_DEST,
    E_BLENDING_NONE
};

struct s_screen_vertex
{
    float x, y, z, rhw;
    float u, v;
    static const DWORD FVF;
};

struct s_vertex
{
    float x, y, z;
    float nx, ny, nz;
};

//-----------------------------------------------------------------------------
// Name: class CMyD3DApplication
// Desc: Application class. The base class (CD3DApplication) provides the
//       generic functionality needed in all Direct3D samples. CMyD3DApplication
//       adds functionality specific to this sample program.
//-----------------------------------------------------------------------------
class CMyD3DApplication : public CD3DApplication
{
private:
    void render_teapot(const D3DXMATRIX &matrix);
    void render_depth_of_field();
    void render_motion_blur();
    void render_text_aux(float offset_x, float offset_y, D3DCOLOR fg);
    void render_stencil();
    void render_stencil_mask();
    void render_stencil_stipple();
    void tile_background();
    void create_texture(LPCTSTR file = NULL);
    void on_texture_file(HWND window, WPARAM wp, LPARAM lp);
    void set_color_write_enable();
    void on_command(HWND window, WPARAM wp, LPARAM lp);

    BOOL                    m_bLoadingApp;          // TRUE, if the app is loading
    CD3DFont*               m_pFont;                // Font for drawing text
    UserInput               m_UserInput;            // Struct for storing user input
    FLOAT                   m_fWorldRotX;           // World rotation state X-axis
    FLOAT                   m_fWorldRotY;           // World rotation state Y-axis
    float m_last_rot_x;
    float m_last_rot_y;

    CComPtr<ID3DXMesh> m_teapot;
    CComPtr<IDirect3DTexture8> m_texture;
    CComPtr<IDirect3DSurface8> m_tile;
    CComPtr<IDirect3DSurface8> m_device_tile;
    CComPtr<ID3DXMesh> m_torus;
    CComPtr<IDirect3DVertexBuffer8> m_stipple_verts;
    CComPtr<IDirect3DTexture8> m_stipple;
    std::vector<RECT> m_tile_rects;
    std::vector<POINT> m_tile_offsets;
    rt::tstring m_texture_file;
    UINT m_num_stipple_quads;
    D3DCOLOR m_text_fg;
    D3DCOLOR m_specular;
    D3DCOLOR m_bg;
    D3DCOLOR m_fg;
    D3DCOLOR m_light_color;
    e_blending_operator m_blending;
    bool m_multisample_motion_blur;
    bool m_multisample_depth_of_field;
    bool m_multisample_antialias;
    bool m_color_write_enable_red;
    bool m_color_write_enable_green;
    bool m_color_write_enable_blue;
    bool m_color_write_enable_alpha;
    bool m_z_write_enable;
    bool m_textured;
    bool m_stencil_stippling;
    bool m_stencil_mask;
    bool m_show_stats;
    bool m_animate_view;
    bool m_dithered;
    bool m_can_texture;
    bool m_destination_alpha;
    bool m_tile_background;
    bool m_build_stencil;

protected:
    HRESULT OneTimeSceneInit();
    HRESULT InitDeviceObjects();
    HRESULT RestoreDeviceObjects();
    HRESULT InvalidateDeviceObjects();
    HRESULT DeleteDeviceObjects();
    HRESULT Render();
    HRESULT FrameMove();
    HRESULT FinalCleanup();
    HRESULT ConfirmDevice( D3DCAPS8*, DWORD, D3DFORMAT );

    HRESULT RenderText();

    void    UpdateInput( UserInput* pUserInput );

    VOID    ReadSettings();
    VOID    WriteSettings();

public:
    LRESULT MsgProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam );
    CMyD3DApplication(BOOL destination_alpha = TRUE,
                      DWORD stencil_bits = 2);
};


#endif // !defined(AFX_RT_FRAMEBUFFER_H__9028BB79_8A32_4ED4_9BE6_FF4CFAA40EFE__INCLUDED_)
